@@ -2,6 +2,7 @@
 
 from AccessControl import getSecurityManager
 from Products.CMFCore.utils import getToolByName
+from Products.CMFPlone.utils import safe_callable
 from datetime import date
 from datetime import datetime
 
@@ -34,6 +35,7 @@ from z3c.form.browser.select import SelectWidget as z3cform_SelectWidget
 from z3c.form.browser.text import TextWidget as z3cform_TextWidget
 from z3c.form.browser.widget import HTMLInputWidget
 from z3c.form.converter import BaseDataConverter
+from z3c.form.converter import SequenceDataConverter
 from z3c.form.converter import CollectionSequenceDataConverter
 from z3c.form.interfaces import IAddForm
 from z3c.form.interfaces import IFieldWidget
@@ -53,11 +55,13 @@ from zope.i18n import translate
 from zope.interface import implementer
 from zope.interface import implements
 from zope.interface import implementsOnly
+from zope.interface import Interface
 from zope.publisher.browser import TestRequest
 from zope.schema.interfaces import IChoice
 from zope.schema.interfaces import ICollection
 from zope.schema.interfaces import IDate
 from zope.schema.interfaces import IDatetime
+from zope.schema.interfaces import IField
 from zope.schema.interfaces import IList
 from zope.schema.interfaces import ISequence
 from zope.security.interfaces import IPermission
@@ -73,6 +77,17 @@ except ImportError:
     HAS_PAC = False
 
 
+try:
+    from z3c.relationfield.interfaces import IRelationChoice
+    from z3c.relationfield.interfaces import IRelationList
+except ImportError:  # pragma: no cover
+    class IRelationChoice(Interface):
+        pass
+
+    class IRelationList(Interface):
+        pass
+
+
 class IDateField(IDate):
     """Marker interface for the DateField."""
 
@@ -181,33 +196,19 @@ class DatetimeWidgetConverter(BaseDataConverter):
         else:
             value += ['00', '00']
 
-        # Eventually set timezone
-        old_val = getattr(self.widget.context, self.field.getName(), None)
-        if getattr(old_val, 'tzinfo', False):
-            # Only set when field was previously set and only if it's value
-            # is a timezone aware datetime object.
-            # effective date for example is timezone naive, at the moment.
-            #
-            # We only ask, if there is a timezone information at all but try to
-            # use the 'timezone' attribute on the widget's context, since it
-            # represents the timezone, the event and it's form input are
-            # related to.
-            timezone = getattr(self.widget.context, 'timezone', None)
-            if timezone:
-                # plone.app.event stores it's timezone on the context.
-                timezone = pytz.timezone(timezone)
-            else:
-                # This better should use the tzinfo object from old_val. But we
-                # have to find, how to correctly localize the value with it.
-                timezone = pytz.utc
-            return timezone.localize(datetime(*map(int, value)))
-        return datetime(*map(int, value))
+        # TODO: respect the selected zone from the widget and just fall back
+        # to default_zone
+        default_zone = self.widget.default_timezone
+        zone = default_zone(self.widget.context)\
+            if safe_callable(default_zone) else default_zone
+        ret = datetime(*map(int, value))
+        if zone:
+            tzinfo = pytz.timezone(zone)
+            ret = tzinfo.localize(ret)
+        return ret
 
 
-class SelectWidgetConverter(CollectionSequenceDataConverter):
-    """Data converter for Select widget."""
-
-    adapts(ICollection, ISelectWidget)
+class SelectWidgetConverterBase(object):
 
     def toFieldValue(self, value):
         """Converts from widget value to field.
@@ -228,11 +229,22 @@ class SelectWidgetConverter(CollectionSequenceDataConverter):
                 return self.field.missing_value
         elif value == (u'',):
             return self.field.missing_value
-        return super(SelectWidgetConverter, self).toFieldValue(value)
+        return super(SelectWidgetConverterBase, self).toFieldValue(value)
+
+
+class SequenceSelectWidgetConverter(
+        SelectWidgetConverterBase, SequenceDataConverter):
+    adapts(IField, ISelectWidget)
+
+
+class SelectWidgetConverter(
+        SelectWidgetConverterBase, CollectionSequenceDataConverter):
+    adapts(ICollection, ISelectWidget)
 
 
 class AjaxSelectWidgetConverter(BaseDataConverter):
-    """Data converter for ICollection."""
+    """Data converter for ICollection fields using the AjaxSelectWidget.
+    """
 
     adapts(ICollection, IAjaxSelectWidget)
 
@@ -272,8 +284,34 @@ class AjaxSelectWidgetConverter(BaseDataConverter):
                               for v in value.split(separator))
 
 
+class RelationChoiceRelatedItemsWidgetConverter(BaseDataConverter):
+    """Data converter for RelationChoice fields using the RelatedItemsWidget.
+    """
+
+    adapts(IRelationChoice, IRelatedItemsWidget)
+
+    def toWidgetValue(self, value):
+        if not value:
+            return self.field.missing_value
+        return IUUID(value)
+
+    def toFieldValue(self, value):
+        if not value:
+            return self.field.missing_value
+        try:
+            catalog = getToolByName(self.widget.context, 'portal_catalog')
+        except AttributeError:
+            catalog = getToolByName(getSite(), 'portal_catalog')
+
+        res = catalog(UID=value)
+        if res:
+            return res[0].getObject()
+        else:
+            return self.field.missing_value
+
+
 class RelatedItemsDataConverter(BaseDataConverter):
-    """Data converter for ICollection."""
+    """Data converter for ICollection fields using the RelatedItemsWidget."""
 
     adapts(ICollection, IRelatedItemsWidget)
 
@@ -289,7 +327,10 @@ class RelatedItemsDataConverter(BaseDataConverter):
         if not value:
             return self.field.missing_value
         separator = getattr(self.widget, 'separator', ';')
-        return separator.join([IUUID(o) for o in value if value])
+        if IRelationList.providedBy(self.field):
+            return separator.join([IUUID(o) for o in value if value])
+        else:
+            return separator.join(v for v in value if v)
 
     def toFieldValue(self, value):
         """Converts from widget value to field.
@@ -300,24 +341,26 @@ class RelatedItemsDataConverter(BaseDataConverter):
         :returns: List of content objects
         :rtype: list | tuple | set
         """
+        if not value:
+            return self.field.missing_value
+
         collectionType = self.field._type
         if isinstance(collectionType, tuple):
             collectionType = collectionType[-1]
 
-        if not len(value):
-            return self.field.missing_value
-
         separator = getattr(self.widget, 'separator', ';')
         value = value.split(separator)
-        value = [v.split('/')[0] for v in value]
 
-        try:
-            catalog = getToolByName(self.widget.context, 'portal_catalog')
-        except AttributeError:
-            catalog = getToolByName(getSite(), 'portal_catalog')
+        if IRelationList.providedBy(self.field):
+            try:
+                catalog = getToolByName(self.widget.context, 'portal_catalog')
+            except AttributeError:
+                catalog = getToolByName(getSite(), 'portal_catalog')
 
-        return collectionType(item.getObject()
-                              for item in catalog(UID=value) if item)
+            return collectionType(item.getObject()
+                                  for item in catalog(UID=value) if item)
+        else:
+            return collectionType(v for v in value)
 
 
 class QueryStringDataConverter(BaseDataConverter):
@@ -458,7 +501,13 @@ class DateWidget(BaseWidget, HTMLInputWidget):
 
 
 class DatetimeWidget(DateWidget, HTMLInputWidget):
-    """Datetime widget for z3c.form."""
+    """Datetime widget for z3c.form.
+
+    :param default_timezone: A Olson DB/pytz timezone identifier or a callback
+                             returning such an identifier.
+    :type default_timezone: String or callback
+
+    """
 
     _converter = DatetimeWidgetConverter
     _formater = 'dateTime'
@@ -467,6 +516,8 @@ class DatetimeWidget(DateWidget, HTMLInputWidget):
 
     pattern_options = DateWidget.pattern_options.copy()
 
+    default_timezone = None
+
     def _base_args(self):
         """Method which will calculate _base class arguments.
 
@@ -557,10 +608,7 @@ class SelectWidget(BaseWidget, z3cform_SelectWidget):
         if (self.name not in self.request and
                 self.name + '-empty-marker' in self.request):
             return []
-        value = self.request.get(self.name, default)
-        if not isinstance(value, (tuple, list)):
-            value = (value,)
-        return value
+        return self.request.get(self.name, default)
 
 
 class AjaxSelectWidget(BaseWidget, z3cform_TextWidget):
@@ -578,11 +626,6 @@ class AjaxSelectWidget(BaseWidget, z3cform_TextWidget):
     vocabulary_view = '@@getVocabulary'
     orderable = False
 
-    def update(self, *args, **kwargs):
-        if not hasattr(self, 'vocabulary'):
-            self.vocabulary = getattr(self.field, 'vocabularyName', None)
-        z3cform_TextWidget.update(self, *args, **kwargs)
-
     def _base_args(self):
         """Method which will calculate _base class arguments.
 
@@ -610,12 +653,27 @@ class AjaxSelectWidget(BaseWidget, z3cform_TextWidget):
         if IAddForm.providedBy(getattr(self, 'form')):
             context = self.form
 
+        vocabulary_name = self.vocabulary
+        field = None
+        if IChoice.providedBy(self.field):
+            args['pattern_options']['maximumSelectionSize'] = 1
+            field = self.field
+        elif ICollection.providedBy(self.field):
+            field = self.field.value_type
+        if not vocabulary_name and field is not None:
+            vocabulary_name = field.vocabularyName
+
         args['pattern_options'] = dict_merge(
             get_ajaxselect_options(context, args['value'], self.separator,
-                                   self.vocabulary, self.vocabulary_view,
+                                   vocabulary_name, self.vocabulary_view,
                                    field_name),
             args['pattern_options'])
 
+        if field and getattr(field, 'vocabulary', None):
+            form_url = self.request.getURL()
+            source_url = "%s/++widget++%s/@@getSource" % (form_url, self.name)
+            args['pattern_options']['vocabularyUrl'] = source_url
+
         # ISequence represents an orderable collection
         if ISequence.providedBy(self.field) or self.orderable:
             args['pattern_options']['orderable'] = True
@@ -638,16 +696,6 @@ class RelatedItemsWidget(BaseWidget, z3cform_TextWidget):
     vocabulary_view = '@@getVocabulary'
     orderable = False
 
-    def update(self, *args, **kwargs):
-        value_type = getattr(self.field, 'value_type', None)
-        if value_type:
-            self.vocabulary = getattr(value_type,
-                                      'vocabularyName',
-                                      'plone.app.vocabularies.Catalog')
-        if self.vocabulary is None:
-            self.vocabulary = 'plone.app.vocabularies.Catalog'
-        z3cform_TextWidget.update(self, *args, **kwargs)
-
     def _base_args(self):
         """Method which will calculate _base class arguments.
 
@@ -664,17 +712,36 @@ class RelatedItemsWidget(BaseWidget, z3cform_TextWidget):
 
         args['name'] = self.name
         args['value'] = self.value
-
         args.setdefault('pattern_options', {})
+
+        field = None
         if IChoice.providedBy(self.field):
             args['pattern_options']['maximumSelectionSize'] = 1
+            field = self.field
+        elif ICollection.providedBy(self.field):
+            field = self.field.value_type
+
+        vocabulary_name = self.vocabulary
+        if not vocabulary_name:
+            if field is not None and field.vocabularyName:
+                vocabulary_name = field.vocabularyName
+            else:
+                vocabulary_name = 'plone.app.vocabularies.Catalog'
+
         field_name = self.field and self.field.__name__ or None
         args['pattern_options'] = dict_merge(
             get_relateditems_options(self.context, args['value'],
-                                     self.separator, self.vocabulary,
+                                     self.separator, vocabulary_name,
                                      self.vocabulary_view, field_name),
             args['pattern_options'])
 
+        if not self.vocabulary:  # widget vocab takes precedence over field
+            if field and getattr(field, 'vocabulary', None):
+                form_url = self.request.getURL()
+                source_url = "%s/++widget++%s/@@getSource" % (
+                    form_url, self.name)
+                args['pattern_options']['vocabularyUrl'] = source_url
+
         return args
 
 
@@ -770,9 +837,16 @@ def DatetimeFieldWidget(field, request):
 
 
 @implementer(IFieldWidget)
-def RelatedItemsFieldWidget(field, request):
-    # TODO: when field is type IRelationChoice configure widget to only allow
-    # one item to be selected
+def AjaxSelectFieldWidget(field, request, extra=None):
+    if extra is not None:
+        request = extra
+    return FieldWidget(field, AjaxSelectWidget(request))
+
+
+@implementer(IFieldWidget)
+def RelatedItemsFieldWidget(field, request, extra=None):
+    if extra is not None:
+        request = extra
     return FieldWidget(field, RelatedItemsWidget(request))
 
 
